/*
 * Copyright (c) 2015 Oracle and/or its affiliates. All rights reserved. This
 * code is released under a tri EPL/GPL/LGPL license. You can use it,
 * redistribute it and/or modify it under the terms of the:
 *
 * Eclipse Public License version 1.0
 * GNU General Public License version 2
 * GNU Lesser General Public License version 2.1
 */
package org.jruby.truffle.format.nodes.type;

import com.oracle.truffle.api.dsl.NodeChild;
import com.oracle.truffle.api.dsl.NodeChildren;
import com.oracle.truffle.api.dsl.Specialization;
import org.jruby.truffle.format.nodes.PackNode;
import org.jruby.truffle.runtime.RubyContext;

/**
 * Re-interpret a value as a {@code long}. In other words, get the raw bytes
 * as a long.
 */
@NodeChildren({
        @NodeChild(value = "value", type = PackNode.class),
})
public abstract class ReinterpretLongNode extends PackNode {

    public ReinterpretLongNode(RubyContext context) {
        super(context);
    }

    @Specialization
    public long asLong(float object) {
        return (long) Float.floatToIntBits(object);
    }

    @Specialization
    public long asLong(double object) {
        return Double.doubleToLongBits(object);
    }

}
